@@ -1,27 +1,30 @@
-#include <filezilla.h>
+#include "../filezilla.h"
 
 #include "cwd.h"
 #include "chmod.h"
 #include "delete.h"
-#include "../directorycache.h"
-#include "directorylistingparser.h"
-#include "engineprivate.h"
-#include "externalipresolver.h"
 #include "filetransfer.h"
 #include "ftpcontrolsocket.h"
-#include "iothread.h"
 #include "list.h"
 #include "logon.h"
 #include "mkd.h"
-#include "pathcache.h"
-#include "proxy.h"
 #include "rawcommand.h"
 #include "rawtransfer.h"
 #include "rename.h"
 #include "rmd.h"
-#include "servercapabilities.h"
 #include "transfersocket.h"
 
+#include "../directorycache.h"
+#include "../directorylistingparser.h"
+#include "../engineprivate.h"
+#include "../iothread.h"
+#include "../pathcache.h"
+#include "../proxy.h"
+#include "../servercapabilities.h"
+
+#include "../../include/externalipresolver.h"
+#include "../../include/engine_options.h"
+
 #include <libfilezilla/file.hpp>
 #include <libfilezilla/iputils.hpp>
 #include <libfilezilla/local_filesys.hpp>
@@ -30,6 +33,8 @@
 
 #include <algorithm>
 
+#include <assert.h>
+
 CFtpControlSocket::CFtpControlSocket(CFileZillaEnginePrivate & engine)
 	: CRealControlSocket(engine)
 {
@@ -93,7 +98,7 @@ void CFtpControlSocket::OnReceive()
 					ParseLine(line);
 
 					// Abort if connection got closed
-					if (!currentServer_) {
+					if (!active_layer_) {
 						return;
 					}
 				}
@@ -174,6 +179,8 @@ void CFtpControlSocket::ParseLine(std::wstring line)
 void CFtpControlSocket::OnConnect()
 {
 	m_lastTypeBinary = -1;
+	m_sentRestartOffset = false;
+	m_protectDataChannel = false;
 
 	SetAlive();
 
@@ -203,7 +210,6 @@ void CFtpControlSocket::OnConnect()
 		log(logmsg::status, _("Connection established, waiting for welcome message..."));
 	}
 	m_pendingReplies = 1;
-	m_repliesToSkip = 0;
 }
 
 void CFtpControlSocket::ParseResponse()
@@ -313,21 +319,6 @@ int CFtpControlSocket::SendCommand(std::wstring const& str, bool maskArgs, bool
 
 void CFtpControlSocket::List(CServerPath const& path, std::wstring const& subDir, int flags)
 {
-	CServerPath newPath = currentPath_;
-	if (!path.empty()) {
-		newPath = path;
-	}
-	if (!newPath.ChangePath(subDir)) {
-		newPath.clear();
-	}
-
-	if (newPath.empty()) {
-		log(logmsg::status, _("Retrieving directory listing..."));
-	}
-	else {
-		log(logmsg::status, _("Retrieving directory listing of \"%s\"..."), newPath.GetPath());
-	}
-
 	Push(std::make_unique<CFtpListOpData>(*this, path, subDir, flags));
 }
 
@@ -432,7 +423,6 @@ void CFtpControlSocket::FileTransfer(std::wstring const& localFile, CServerPath
 	log(logmsg::debug_verbose, L"CFtpControlSocket::FileTransfer()");
 
 	auto pData = std::make_unique<CFtpFileTransferOpData>(*this, download, localFile, remoteFile, remotePath, transferSettings);
-	pData->binary = transferSettings.binary;
 	Push(std::move(pData));
 }
 
@@ -506,14 +496,12 @@ bool CFtpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotific
 				return false;
 			}
 
-			auto & data = static_cast<CFtpLogonOpData&>(*operations_.back());
-
 			CInteractiveLoginNotification *pInteractiveLoginNotification = static_cast<CInteractiveLoginNotification *>(pNotification);
 			if (!pInteractiveLoginNotification->passwordSet) {
 				ResetOperation(FZ_REPLY_CANCELED);
 				return false;
 			}
-			data.credentials_.SetPass(pInteractiveLoginNotification->credentials.GetPass());
+			credentials_.SetPass(pInteractiveLoginNotification->credentials.GetPass());
 			SendNextCommand();
 		}
 		break;
@@ -588,15 +576,6 @@ void CFtpControlSocket::RemoveDir(CServerPath const& path, std::wstring const& s
 
 void CFtpControlSocket::Mkdir(CServerPath const& path)
 {
-	/* Directory creation works like this: First find a parent directory into
-	 * which we can CWD, then create the subdirs one by one. If either part
-	 * fails, try MKD with the full path directly.
-	 */
-
-	if (operations_.empty() && !path.empty()) {
-		log(logmsg::status, _("Creating directory '%s'..."), path.GetPath());
-	}
-
 	auto pData = std::make_unique<CFtpMkdirOpData>(*this);
 	pData->path_ = path;
 
@@ -605,15 +584,11 @@ void CFtpControlSocket::Mkdir(CServerPath const& path)
 
 void CFtpControlSocket::Rename(CRenameCommand const& command)
 {
-	log(logmsg::status, _("Renaming '%s' to '%s'"), command.GetFromPath().FormatFilename(command.GetFromFile()), command.GetToPath().FormatFilename(command.GetToFile()));
-
 	Push(std::make_unique<CFtpRenameOpData>(*this, command));
 }
 
 void CFtpControlSocket::Chmod(CChmodCommand const& command)
 {
-	log(logmsg::status, _("Setting permissions of '%s' to '%s'"), command.GetPath().FormatFilename(command.GetFile()), command.GetPermission());
-
 	Push(std::make_unique<CFtpChmodOpData>(*this, command));
 }
 
@@ -622,17 +597,17 @@ int CFtpControlSocket::GetExternalIPAddress(std::string& address)
 	// Local IP should work. Only a complete moron would use IPv6
 	// and NAT at the same time.
 	if (socket_->address_family() != fz::address_type::ipv6) {
-		int mode = engine_.GetOptions().GetOptionVal(OPTION_EXTERNALIPMODE);
+		int mode = engine_.GetOptions().get_int(OPTION_EXTERNALIPMODE);
 
 		if (mode) {
-			if (engine_.GetOptions().GetOptionVal(OPTION_NOEXTERNALONLOCAL) &&
+			if (engine_.GetOptions().get_int(OPTION_NOEXTERNALONLOCAL) &&
 				!fz::is_routable_address(socket_->peer_ip()))
 				// Skip next block, use local address
 				goto getLocalIP;
 		}
 
 		if (mode == 1) {
-			std::wstring ip = engine_.GetOptions().GetOption(OPTION_EXTERNALIP);
+			std::wstring ip = engine_.GetOptions().get_string(OPTION_EXTERNALIP);
 			if (!ip.empty()) {
 				address = fz::to_string(ip);
 				return FZ_REPLY_OK;
@@ -644,14 +619,14 @@ int CFtpControlSocket::GetExternalIPAddress(std::string& address)
 			if (!m_pIPResolver) {
 				std::string localAddress = socket_->local_ip(true);
 
-				if (!localAddress.empty() && localAddress == fz::to_string(engine_.GetOptions().GetOption(OPTION_LASTRESOLVEDIP))) {
+				if (!localAddress.empty() && localAddress == fz::to_string(engine_.GetOptions().get_string(OPTION_LASTRESOLVEDIP))) {
 					log(logmsg::debug_verbose, L"Using cached external IP address");
 
 					address = localAddress;
 					return FZ_REPLY_OK;
 				}
 
-				std::wstring resolverAddress = engine_.GetOptions().GetOption(OPTION_EXTERNALIPRESOLVER);
+				std::wstring resolverAddress = engine_.GetOptions().get_string(OPTION_EXTERNALIPRESOLVER);
 
 				log(logmsg::debug_info, _("Retrieving external IP address from %s"), resolverAddress);
 
@@ -671,7 +646,7 @@ int CFtpControlSocket::GetExternalIPAddress(std::string& address)
 				log(logmsg::debug_info, L"Got external IP address");
 				address = m_pIPResolver->GetIP();
 
-				engine_.GetOptions().SetOption(OPTION_LASTRESOLVEDIP, fz::to_wstring(address));
+				engine_.GetOptions().set(OPTION_LASTRESOLVEDIP, fz::to_wstring(address));
 
 				m_pIPResolver.reset();
 
@@ -712,39 +687,6 @@ void CFtpControlSocket::Transfer(std::wstring const& cmd, CFtpTransferOpData* ol
 	pData->pOldData = oldData;
 	pData->pOldData->transferEndReason = TransferEndReason::successful;
 
-	if (proxy_layer_) {
-		// Only passive suported
-		// Theoretically could use reverse proxy ability in SOCKS5, but
-		// it is too fragile to set up with all those broken routers and
-		// firewalls sabotaging connections. Regular active mode is hard
-		// enough already
-		pData->bPasv = true;
-		pData->bTriedActive = true;
-	}
-	else {
-		switch (currentServer_.GetPasvMode())
-		{
-		case MODE_PASSIVE:
-			pData->bPasv = true;
-			break;
-		case MODE_ACTIVE:
-			pData->bPasv = false;
-			break;
-		default:
-			pData->bPasv = engine_.GetOptions().GetOptionVal(OPTION_USEPASV) != 0;
-			break;
-		}
-	}
-
-	if ((pData->pOldData->binary && m_lastTypeBinary == 1) ||
-		(!pData->pOldData->binary && m_lastTypeBinary == 0))
-	{
-		pData->opState = rawtransfer_port_pasv;
-	}
-	else {
-		pData->opState = rawtransfer_type;
-	}
-
 	Push(std::move(pData));
 }
 
@@ -756,8 +698,9 @@ void CFtpControlSocket::Connect(CServer const& server, Credentials const& creden
 	}
 
 	currentServer_ = server;
+	credentials_ = credentials;
 
-	Push(std::make_unique<CFtpLogonOpData>(*this, credentials));
+	Push(std::make_unique<CFtpLogonOpData>(*this));
 }
 
 void CFtpControlSocket::OnTimer(fz::timer_id id)
@@ -805,7 +748,7 @@ void CFtpControlSocket::OnTimer(fz::timer_id id)
 
 void CFtpControlSocket::StartKeepaliveTimer()
 {
-	if (!engine_.GetOptions().GetOptionVal(OPTION_FTP_SENDKEEPALIVE)) {
+	if (!engine_.GetOptions().get_int(OPTION_FTP_SENDKEEPALIVE)) {
 		return;
 	}
 
@@ -851,6 +794,12 @@ void CFtpControlSocket::ResetSocket()
 {
 	receiveBuffer_.clear();
 	tls_layer_.reset();
+	m_pendingReplies = 0;
+	m_repliesToSkip = 0;
+	m_Response.clear();
+	m_MultilineResponseCode.clear();;
+	m_MultilineResponseLines.clear();
+
 	CRealControlSocket::ResetSocket();
 }
 
@@ -860,5 +809,17 @@ void CFtpControlSocket::OnVerifyCert(fz::tls_layer* source, fz::tls_session_info
 		return;
 	}
 
-	SendAsyncRequest(new CCertificateNotification(std::move(info)));
+	SendAsyncRequest(std::make_unique<CCertificateNotification>(std::move(info)));
+}
+
+void CFtpControlSocket::Push(std::unique_ptr<COpData> && pNewOpData)
+{
+	CRealControlSocket::Push(std::move(pNewOpData));
+	if (operations_.size() == 1 && operations_.back()->opId != Command::connect) {
+		if (!active_layer_) {
+			std::unique_ptr<COpData> connOp = std::make_unique<CFtpLogonOpData>(*this);
+			connOp->topLevelOperation_ = true;
+			CRealControlSocket::Push(std::move(connOp));
+		}
+	}
 }
