@@ -1,32 +1,33 @@
-#include <filezilla.h>
+#include "../filezilla.h"
 
 #include "connect.h"
-#include "ControlSocket.h"
-#include "engineprivate.h"
 #include "filetransfer.h"
 #include "httpcontrolsocket.h"
 #include "internalconnect.h"
 #include "request.h"
-#include "socket_errors.h"
-#include "tlssocket.h"
+
+#include "../controlsocket.h"
+#include "../engineprivate.h"
 
 #include <libfilezilla/file.hpp>
 #include <libfilezilla/iputils.hpp>
 #include <libfilezilla/local_filesys.hpp>
+#include <libfilezilla/tls_layer.hpp>
 #include <libfilezilla/uri.hpp>
 
+#include <assert.h>
 #include <string.h>
 
 int simple_body::data_request(unsigned char* data, unsigned int & len)
 {
-	len = std::min(static_cast<size_t>(len), body_.size() - written_);
+	len = static_cast<unsigned int>(std::min(static_cast<size_t>(len), body_.size() - written_));
 	memcpy(data, body_.c_str() + written_, len);
 	written_ += len;
 	return FZ_REPLY_CONTINUE;
 }
 
 
-file_body::file_body(fz::file & file, uint64_t start, uint64_t size, CLogging & logger)
+file_body::file_body(fz::file & file, uint64_t start, uint64_t size, fz::logger_interface & logger)
 	: file_(file)
 	, start_(start)
 	, size_(size)
@@ -39,10 +40,13 @@ int file_body::data_request(unsigned char* data, unsigned int & len)
 	assert(size_ >= written_);
 	assert(len > 0);
 	len = static_cast<unsigned int>(std::min(static_cast<uint64_t>(len), size_ - written_));
+	if (!len) {
+		return FZ_REPLY_CONTINUE;
+	}
 	auto bytes_read = file_.read(data, len);
 	if (bytes_read < 0) {
 		len = 0;
-		logger_.LogMessage(MessageType::Error, _("Reading from local file failed"));
+		logger_.log(logmsg::error, _("Reading from local file failed"));
 		return FZ_REPLY_ERROR;
 	}
 	else if (bytes_read == 0) {
@@ -69,10 +73,10 @@ int file_body::rewind()
 	int64_t s = static_cast<int64_t>(start_);
 	if (file_.seek(s, fz::file::begin) != s) {
 		if (!start_) {
-			logger_.LogMessage(MessageType::Error, _("Could not seek to the beginning of the file"));
+			logger_.log(logmsg::error, _("Could not seek to the beginning of the file"));
 		}
 		else {
-			logger_.LogMessage(MessageType::Error, _("Could not seek to offset %d within file"), start_);
+			logger_.log(logmsg::error, _("Could not seek to offset %d within file"), start_);
 		}
 		return FZ_REPLY_ERROR;
 	}
@@ -103,6 +107,63 @@ int HttpResponse::reset()
 	return FZ_REPLY_CONTINUE;
 }
 
+void RequestThrottler::throttle(std::string const& hostname, fz::datetime const& backoff)
+{
+	if (hostname.empty() || !backoff) {
+		return;
+	}
+
+	fz::scoped_lock l(mtx_);
+
+	bool found{};
+	auto now = fz::datetime::now();
+	for (size_t i = 0; i < backoff_.size(); ) {
+		auto & entry = backoff_[i];
+		if (entry.first == hostname) {
+			found = true;
+			if (entry.second < backoff) {
+				entry.second = backoff;
+			}
+		}
+		if (entry.second < now) {
+			backoff_[i] = std::move(backoff_.back());
+			backoff_.pop_back();
+		}
+		else {
+			++i;
+		}
+	}
+	if (!found) {
+		backoff_.emplace_back(hostname, backoff);
+	}
+}
+
+fz::duration RequestThrottler::get_throttle(std::string const& hostname)
+{
+	fz::scoped_lock l(mtx_);
+
+	fz::duration ret;
+
+	auto now = fz::datetime::now();
+	for (size_t i = 0; i < backoff_.size(); ) {
+		auto & entry = backoff_[i];
+		if (entry.second < now) {
+			backoff_[i] = std::move(backoff_.back());
+			backoff_.pop_back();
+		}
+		else {
+			if (entry.first == hostname) {
+				ret = entry.second - now;
+			}
+			++i;
+		}
+	}
+
+	return ret;
+}
+
+RequestThrottler CHttpControlSocket::throttler_;
+
 CHttpControlSocket::CHttpControlSocket(CFileZillaEnginePrivate & engine)
 	: CRealControlSocket(engine)
 {
@@ -116,18 +177,14 @@ CHttpControlSocket::~CHttpControlSocket()
 
 bool CHttpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotification)
 {
-	if (operations_.empty() || !operations_.back()->waitForAsyncRequest) {
-		LogMessage(MessageType::Debug_Info, L"Not waiting for request reply, ignoring request reply %d", pNotification->GetRequestID());
-	}
-
-	operations_.back()->waitForAsyncRequest = false;
+	log(logmsg::debug_verbose, L"CHttpControlSocket::SetAsyncRequestReply");
 
 	switch (pNotification->GetRequestID())
 	{
 	case reqId_fileexists:
 		{
 			if (operations_.back()->opId != Command::transfer) {
-				LogMessage(MessageType::Debug_Info, L"No or invalid operation in progress, ignoring request reply %f", pNotification->GetRequestID());
+				log(logmsg::debug_info, L"No or invalid operation in progress, ignoring request reply %f", pNotification->GetRequestID());
 				return false;
 			}
 
@@ -137,17 +194,17 @@ bool CHttpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 		break;
 	case reqId_certificate:
 		{
-			if (!m_pTlsSocket || m_pTlsSocket->GetState() != CTlsSocket::TlsState::verifycert) {
-				LogMessage(MessageType::Debug_Info, L"No or invalid operation in progress, ignoring request reply %d", pNotification->GetRequestID());
+			if (!tls_layer_ || tls_layer_->get_state() != fz::socket_state::connecting) {
+				log(logmsg::debug_info, L"No or invalid operation in progress, ignoring request reply %d", pNotification->GetRequestID());
 				return false;
 			}
 
 			CCertificateNotification* pCertificateNotification = static_cast<CCertificateNotification *>(pNotification);
-			m_pTlsSocket->TrustCurrentCert(pCertificateNotification->m_trusted);
+			tls_layer_->set_verification_result(pCertificateNotification->trusted_);
 		}
 		break;
 	default:
-		LogMessage(MessageType::Debug_Warning, L"Unknown request %d", pNotification->GetRequestID());
+		log(logmsg::debug_warning, L"Unknown request %d", pNotification->GetRequestID());
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return false;
 	}
@@ -161,19 +218,19 @@ void CHttpControlSocket::OnReceive()
 	if (operations_.empty() || operations_.back()->opId != PrivCommand::http_request) {
 		uint8_t buffer;
 		int error{};
-		int read = m_pBackend->Read(&buffer, 1, error);
+		int read = active_layer_->read(&buffer, 1, error);
 		if (!read) {
-			LogMessage(MessageType::Debug_Warning, L"Idle socket got closed");
+			log(logmsg::debug_warning, L"Idle socket got closed");
 			ResetSocket();
 		}
 		else if (read == -1) {
 			if (error != EAGAIN) {
-				LogMessage(MessageType::Debug_Warning, L"OnReceive called while not processing http request. Reading fails with error %d, closing socket.", error);
+				log(logmsg::debug_warning, L"OnReceive called while not processing http request. Reading fails with error %d, closing socket.", error);
 				ResetSocket();
 			}
 		}
 		else if (read) {
-			LogMessage(MessageType::Debug_Warning, L"Server sent data while not in an active HTTP request, closing socket.");
+			log(logmsg::debug_warning, L"Server sent data while not in an active HTTP request, closing socket.");
 			ResetSocket();
 		}
 		return;
@@ -191,38 +248,32 @@ void CHttpControlSocket::OnReceive()
 void CHttpControlSocket::OnConnect()
 {
 	if (operations_.empty() || operations_.back()->opId != PrivCommand::http_connect) {
-		LogMessage(MessageType::Debug_Warning, L"Discarding stale OnConnect");
+		log(logmsg::debug_warning, L"Discarding stale OnConnect");
 		return;
 	}
 
+	socket_->set_flags(fz::socket::flag_nodelay, true);
+
 	auto & data = static_cast<CHttpInternalConnectOpData &>(*operations_.back());
 
 	if (data.tls_) {
-		if (!m_pTlsSocket) {
-			LogMessage(MessageType::Status, _("Connection established, initializing TLS..."));
+		if (!tls_layer_) {
+			log(logmsg::status, _("Connection established, initializing TLS..."));
 
-			delete m_pBackend;
-			m_pTlsSocket = new CTlsSocket(this, *socket_, this);
-			m_pBackend = m_pTlsSocket;
+			tls_layer_ = std::make_unique<fz::tls_layer>(event_loop_, this, *active_layer_, &engine_.GetContext().GetTlsSystemTrustStore(), logger_);
+			active_layer_ = tls_layer_.get();
 
-			if (!m_pTlsSocket->Init()) {
-				LogMessage(MessageType::Error, _("Failed to initialize TLS."));
-				DoClose();
-				return;
-			}
-
-			int res = m_pTlsSocket->Handshake();
-			if (res == FZ_REPLY_ERROR) {
+			if (!tls_layer_->client_handshake(&data)) {
 				DoClose();
 			}
 		}
 		else {
-			LogMessage(MessageType::Status, _("TLS connection established, sending HTTP request"));
+			log(logmsg::status, _("TLS connection established, sending HTTP request"));
 			ResetOperation(FZ_REPLY_OK);
 		}
 	}
 	else {
-		LogMessage(MessageType::Status, _("Connection established, sending HTTP request"));
+		log(logmsg::status, _("Connection established, sending HTTP request"));
 		ResetOperation(FZ_REPLY_OK);
 	}
 }
@@ -231,10 +282,10 @@ void CHttpControlSocket::FileTransfer(std::wstring const& localFile, CServerPath
 									std::wstring const& remoteFile, bool download,
 									CFileTransferCommand::t_transferSettings const& settings)
 {
-	LogMessage(MessageType::Debug_Verbose, L"CHttpControlSocket::FileTransfer()");
+	log(logmsg::debug_verbose, L"CHttpControlSocket::FileTransfer()");
 
 	if (download) {
-		LogMessage(MessageType::Status, _("Downloading %s"), remotePath.FormatFilename(remoteFile));
+		log(logmsg::status, _("Downloading %s"), remotePath.FormatFilename(remoteFile));
 	}
 
 	Push(std::make_unique<CHttpFileTransferOpData>(*this, download, localFile, remoteFile, remotePath, settings));
@@ -242,16 +293,21 @@ void CHttpControlSocket::FileTransfer(std::wstring const& localFile, CServerPath
 
 void CHttpControlSocket::FileTransfer(CHttpRequestCommand const& command)
 {
-	LogMessage(MessageType::Debug_Verbose, L"CHttpControlSocket::FileTransfer()");
+	log(logmsg::debug_verbose, L"CHttpControlSocket::FileTransfer()");
 
-	LogMessage(MessageType::Status, _("Requesting %s"), command.uri_.to_string());
+	log(logmsg::status, _("Requesting %s"), command.uri_.to_string());
 
 	Push(std::make_unique<CHttpFileTransferOpData>(*this, command.uri_, command.verb_, command.body_));
 }
 
 void CHttpControlSocket::Request(std::shared_ptr<HttpRequestResponseInterface> const& request)
 {
-	LogMessage(MessageType::Debug_Verbose, L"CHttpControlSocket::Request()");
+	log(logmsg::debug_verbose, L"CHttpControlSocket::Request()");
+
+	if (!request) {
+		log(logmsg::debug_warning, L"Dropping null request");
+		return;
+	}
 
 	auto op = dynamic_cast<CHttpRequestOpData*>(operations_.empty() ? nullptr : operations_.back().get());
 	if (op) {
@@ -264,21 +320,21 @@ void CHttpControlSocket::Request(std::shared_ptr<HttpRequestResponseInterface> c
 
 void CHttpControlSocket::Request(std::deque<std::shared_ptr<HttpRequestResponseInterface>> && requests)
 {
-	LogMessage(MessageType::Debug_Verbose, L"CHttpControlSocket::Request()");
+	log(logmsg::debug_verbose, L"CHttpControlSocket::Request()");
 	Push(std::make_unique<CHttpRequestOpData>(*this, std::move(requests)));
 }
 
 int CHttpControlSocket::InternalConnect(std::wstring const& host, unsigned short port, bool tls, bool allowDisconnect)
 {
-	LogMessage(MessageType::Debug_Verbose, L"CHttpControlSocket::InternalConnect()");
+	log(logmsg::debug_verbose, L"CHttpControlSocket::InternalConnect()");
 
 	if (!Connected()) {
 		return FZ_REPLY_INTERNALERROR;
 	}
 
-	if (m_pBackend) {
+	if (active_layer_) {
 		if (host == connected_host_ && port == connected_port_ && tls == connected_tls_) {
-			LogMessage(MessageType::Debug_Verbose, L"Reusing an existing connection");
+			log(logmsg::debug_verbose, L"Reusing an existing connection");
 			return FZ_REPLY_OK;
 		}
 		if (!allowDisconnect) {
@@ -297,27 +353,25 @@ int CHttpControlSocket::InternalConnect(std::wstring const& host, unsigned short
 
 void CHttpControlSocket::OnSocketError(int error)
 {
-	LogMessage(MessageType::Debug_Verbose, L"CHttpControlSocket::OnClose(%d)", error);
+	log(logmsg::debug_verbose, L"CHttpControlSocket::OnClose(%d)", error);
 
 	if (operations_.empty() || (operations_.back()->opId != PrivCommand::http_connect && operations_.back()->opId != PrivCommand::http_request)) {
-		LogMessage(MessageType::Debug_Warning, L"Idle socket got closed");
+		log(logmsg::debug_warning, L"Idle socket got closed");
 		ResetSocket();
 		return;
 	}
 
-	LogMessage(MessageType::Error, _("Disconnected from server: %s"), fz::socket_error_description(error));
+	log(logmsg::error, _("Disconnected from server: %s"), fz::socket_error_description(error));
 	ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
 }
 
 void CHttpControlSocket::ResetSocket()
 {
-	LogMessage(MessageType::Debug_Verbose, L"CHttpControlSocket::ResetSocket()");
-	if (m_pTlsSocket) {
-		if (m_pTlsSocket != m_pBackend) {
-			delete m_pTlsSocket;
-		}
-		m_pTlsSocket = nullptr;
-	}
+	log(logmsg::debug_verbose, L"CHttpControlSocket::ResetSocket()");
+
+	active_layer_ = nullptr;
+
+	tls_layer_.reset();
 
 	CRealControlSocket::ResetSocket();
 }
@@ -328,9 +382,10 @@ int CHttpControlSocket::Disconnect()
 	return FZ_REPLY_OK;
 }
 
-void CHttpControlSocket::Connect(CServer const& server, Credentials const&)
+void CHttpControlSocket::Connect(CServer const& server, Credentials const& credentials)
 {
 	currentServer_ = server;
+	credentials_ = credentials;
 	Push(std::make_unique<CHttpConnectOpData>(*this));
 }
 
