@@ -1,10 +1,13 @@
-#include <filezilla.h>
+#include "../filezilla.h"
 
 #include "connect.h"
 #include "event.h"
 #include "input_thread.h"
-#include "proxy.h"
+#include "../proxy.h"
 
+#include "../../include/engine_options.h"
+
+#include <libfilezilla/local_filesys.hpp>
 #include <libfilezilla/process.hpp>
 
 int CSftpConnectOpData::Send()
@@ -13,16 +16,39 @@ int CSftpConnectOpData::Send()
 	{
 	case connect_init:
 		{
-			auto executable = fz::to_native(engine_.GetOptions().GetOption(OPTION_FZSFTP_EXECUTABLE));
+			log(logmsg::status, _("Connecting to %s..."), currentServer_.Format(ServerFormat::with_optional_port, controlSocket_.credentials_));
+
+			if (!controlSocket_.credentials_.keyFile_.empty()) {
+				keyfiles_ = fz::strtok(controlSocket_.credentials_.keyFile_, L"\r\n");
+			}
+			else {
+				keyfiles_ = fz::strtok(engine_.GetOptions().get_string(OPTION_SFTP_KEYFILES), L"\r\n");
+			}
+
+			keyfiles_.erase(
+						std::remove_if(keyfiles_.begin(), keyfiles_.end(),
+									   [this](std::wstring const& keyfile) {
+							if (fz::local_filesys::get_file_type(fz::to_native(keyfile), true) != fz::local_filesys::file) {
+								log(logmsg::status, _("Skipping non-existing key file \"%s\""), keyfile);
+								return true;
+							}
+							return false;
+						}), keyfiles_.end());
+
+			keyfile_ = keyfiles_.cbegin();
+
+			auto executable = fz::to_native(engine_.GetOptions().get_string(OPTION_FZSFTP_EXECUTABLE));
 			if (executable.empty()) {
 				executable = fzT("fzsftp");
 			}
 			log(logmsg::debug_verbose, L"Going to execute %s", executable);
 
 			std::vector<fz::native_string> args = { fzT("-v") };
-			if (engine_.GetOptions().GetOptionVal(OPTION_SFTP_COMPRESSION)) {
+			if (engine_.GetOptions().get_int(OPTION_SFTP_COMPRESSION)) {
 				args.push_back(fzT("-C"));
 			}
+			engine_.GetRateLimiter().add(&controlSocket_);
+			controlSocket_.process_ = std::make_unique<fz::process>();
 			if (!controlSocket_.process_->spawn(executable, args)) {
 				log(logmsg::debug_warning, L"Could not create process");
 				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
@@ -39,7 +65,7 @@ int CSftpConnectOpData::Send()
 	case connect_proxy:
 		{
 			int type;
-			switch (engine_.GetOptions().GetOptionVal(OPTION_PROXY_TYPE))
+			switch (engine_.GetOptions().get_int(OPTION_PROXY_TYPE))
 			{
 			case static_cast<int>(ProxyType::HTTP):
 				type = 1;
@@ -56,15 +82,15 @@ int CSftpConnectOpData::Send()
 			}
 
 			std::wstring cmd = fz::sprintf(L"proxy %d \"%s\" %d", type,
-				engine_.GetOptions().GetOption(OPTION_PROXY_HOST),
-				engine_.GetOptions().GetOptionVal(OPTION_PROXY_PORT));
-			std::wstring user = engine_.GetOptions().GetOption(OPTION_PROXY_USER);
+				engine_.GetOptions().get_string(OPTION_PROXY_HOST),
+				engine_.GetOptions().get_int(OPTION_PROXY_PORT));
+			std::wstring user = engine_.GetOptions().get_string(OPTION_PROXY_USER);
 			if (!user.empty()) {
 				cmd += L" \"" + user + L"\"";
 			}
 
 			std::wstring show = cmd;
-			std::wstring pass = engine_.GetOptions().GetOption(OPTION_PROXY_PASS);
+			std::wstring pass = engine_.GetOptions().get_string(OPTION_PROXY_PASS);
 			if (!pass.empty()) {
 				cmd += L" \"" + pass + L"\"";
 				show += L" \"" + std::wstring(pass.size(), '*') + L"\"";
@@ -76,7 +102,7 @@ int CSftpConnectOpData::Send()
 		return controlSocket_.SendCommand(L"keyfile \"" + *(keyfile_++) + L"\"");
 	case connect_open:
 		{
-			std::wstring user = (credentials_.logonType_ == LogonType::anonymous) ? L"anonymous" : currentServer_.GetUser();
+			std::wstring user = (controlSocket_.credentials_.logonType_ == LogonType::anonymous) ? L"anonymous" : currentServer_.GetUser();
 			return controlSocket_.SendCommand(fz::sprintf(L"open \"%s@%s\" %d", user, controlSocket_.ConvertDomainName(currentServer_.GetHost()), currentServer_.GetPort()));
 		}
 	default:
@@ -100,7 +126,7 @@ int CSftpConnectOpData::ParseResponse()
 			log(logmsg::error, _("fzsftp belongs to a different version of FileZilla"));
 			return FZ_REPLY_INTERNALERROR | FZ_REPLY_DISCONNECTED;
 		}
-		if (engine_.GetOptions().GetOptionVal(OPTION_PROXY_TYPE) && !currentServer_.GetBypassProxy()) {
+		if (engine_.GetOptions().get_int(OPTION_PROXY_TYPE) && !currentServer_.GetBypassProxy()) {
 			opState = connect_proxy;
 		}
 		else if (keyfile_ != keyfiles_.cend()) {
@@ -124,7 +150,7 @@ int CSftpConnectOpData::ParseResponse()
 		}
 		break;
 	case connect_open:
-		engine_.AddNotification(new CSftpEncryptionNotification(controlSocket_.m_sftpEncryptionDetails));
+		engine_.AddNotification(std::make_unique<CSftpEncryptionNotification>(controlSocket_.m_sftpEncryptionDetails));
 		return FZ_REPLY_OK;
 	default:
 		log(logmsg::debug_warning, L"Unknown op state: %d", opState);
