@@ -1,10 +1,14 @@
-#include <filezilla.h>
+#include "../filezilla.h"
 
 #include "../directorycache.h"
 #include "filetransfer.h"
 
+#include "../../include/engine_options.h"
+
 #include <libfilezilla/local_filesys.hpp>
 
+#include <assert.h>
+
 enum filetransferStates
 {
 	filetransfer_init = 0,
@@ -133,7 +137,7 @@ int CSftpFileTransferOpData::Send()
 int CSftpFileTransferOpData::ParseResponse()
 {
 	if (opState == filetransfer_transfer) {
-		if (controlSocket_.result_ == FZ_REPLY_OK && engine_.GetOptions().GetOptionVal(OPTION_PRESERVE_TIMESTAMPS)) {
+		if (controlSocket_.result_ == FZ_REPLY_OK && engine_.GetOptions().get_int(OPTION_PRESERVE_TIMESTAMPS)) {
 			if (download_) {
 				if (!fileTime_.empty()) {
 					if (!fz::local_filesys::set_modification_time(fz::to_native(localFile_), fileTime_))
@@ -204,7 +208,7 @@ int CSftpFileTransferOpData::SubcommandResult(int prevResult, COpData const&)
 				if (!dirDidExist) {
 					opState = filetransfer_waitlist;
 				}
-				else if (download_ && engine_.GetOptions().GetOptionVal(OPTION_PRESERVE_TIMESTAMPS)) {
+				else if (download_ && engine_.GetOptions().get_int(OPTION_PRESERVE_TIMESTAMPS)) {
 					opState = filetransfer_mtime;
 				}
 				else {
@@ -223,7 +227,7 @@ int CSftpFileTransferOpData::SubcommandResult(int prevResult, COpData const&)
 						}
 
 						if (download_ && !entry.has_time() &&
-							engine_.GetOptions().GetOptionVal(OPTION_PRESERVE_TIMESTAMPS))
+							engine_.GetOptions().get_int(OPTION_PRESERVE_TIMESTAMPS))
 						{
 							opState = filetransfer_mtime;
 						}
@@ -263,7 +267,7 @@ int CSftpFileTransferOpData::SubcommandResult(int prevResult, COpData const&)
 					opState = filetransfer_mtime;
 				}
 				else if (download_ &&
-					engine_.GetOptions().GetOptionVal(OPTION_PRESERVE_TIMESTAMPS))
+					engine_.GetOptions().get_int(OPTION_PRESERVE_TIMESTAMPS))
 				{
 					opState = filetransfer_mtime;
 				}
@@ -279,7 +283,7 @@ int CSftpFileTransferOpData::SubcommandResult(int prevResult, COpData const&)
 					}
 
 					if (download_ && !entry.has_time() &&
-						engine_.GetOptions().GetOptionVal(OPTION_PRESERVE_TIMESTAMPS))
+						engine_.GetOptions().get_int(OPTION_PRESERVE_TIMESTAMPS))
 					{
 						opState = filetransfer_mtime;
 					}
