@@ -1,20 +1,18 @@
-#include <filezilla.h>
+#include "../filezilla.h"
 
 #include "connect.h"
 #include "delete.h"
 #include "event.h"
 #include "input_thread.h"
 #include "../directorycache.h"
-#include "directorylistingparser.h"
-#include "engineprivate.h"
+#include "../directorylistingparser.h"
+#include "../engineprivate.h"
 #include "file_transfer.h"
 #include "list.h"
 #include "mkd.h"
-#include "pathcache.h"
-#include "proxy.h"
-#include "resolve.h"
+#include "../proxy.h"
 #include "rmd.h"
-#include "servercapabilities.h"
+#include "../servercapabilities.h"
 #include "storjcontrolsocket.h"
 
 #include <libfilezilla/event_loop.hpp>
@@ -23,7 +21,8 @@
 #include <libfilezilla/thread_pool.hpp>
 
 #include <algorithm>
-#include <cwchar>
+
+#include <assert.h>
 
 CStorjControlSocket::CStorjControlSocket(CFileZillaEnginePrivate & engine)
 	: CControlSocket(engine)
@@ -39,34 +38,15 @@ CStorjControlSocket::~CStorjControlSocket()
 
 void CStorjControlSocket::Connect(CServer const &server, Credentials const& credentials)
 {
-	log(logmsg::status, _("Connecting to %s..."), server.Format(ServerFormat::with_optional_port));
-	SetWait(true);
-
 	currentServer_ = server;
+	credentials_ = credentials;
 
-	process_ = std::make_unique<fz::process>();
-
-	Push(std::make_unique<CStorjConnectOpData>(*this, credentials));
+	Push(std::make_unique<CStorjConnectOpData>(*this));
 }
 
 void CStorjControlSocket::List(CServerPath const& path, std::wstring const& subDir, int flags)
 {
-	CServerPath newPath = currentPath_;
-	if (!path.empty()) {
-		newPath = path;
-	}
-	if (!newPath.ChangePath(subDir)) {
-		newPath.clear();
-	}
-
-	if (newPath.empty()) {
-		log(logmsg::status, _("Retrieving directory listing..."));
-	}
-	else {
-		log(logmsg::status, _("Retrieving directory listing of \"%s\"..."), newPath.GetPath());
-	}
-
-	Push(std::make_unique<CStorjListOpData>(*this, newPath, std::wstring(), flags));
+	Push(std::make_unique<CStorjListOpData>(*this, path, subDir, flags));
 }
 
 void CStorjControlSocket::FileTransfer(std::wstring const& localFile, CServerPath const& remotePath,
@@ -88,22 +68,8 @@ void CStorjControlSocket::Delete(CServerPath const& path, std::vector<std::wstri
 	Push(std::make_unique<CStorjDeleteOpData>(*this, path, std::move(files)));
 }
 
-void CStorjControlSocket::Resolve(CServerPath const& path, std::wstring const& file, std::wstring & bucket, std::wstring * fileId, bool ignore_missing_file)
-{
-	Push(std::make_unique<CStorjResolveOpData>(*this, path, file, bucket, fileId, ignore_missing_file));
-}
-
-void CStorjControlSocket::Resolve(CServerPath const& path, std::vector<std::wstring> const& files, std::wstring & bucket, std::vector<std::wstring> & fileIds)
-{
-	Push(std::make_unique<CStorjResolveManyOpData>(*this, path, files, bucket, fileIds));
-}
-
 void CStorjControlSocket::Mkdir(CServerPath const& path)
 {
-	if (operations_.empty()) {
-		log(logmsg::status, _("Creating directory '%s'..."), path.GetPath());
-	}
-
 	auto pData = std::make_unique<CStorjMkdirOpData>(*this);
 	pData->path_ = path;
 	Push(std::move(pData));
@@ -168,7 +134,7 @@ void CStorjControlSocket::OnStorjEvent(storj_message const& message)
 			break;
 		}
 		else {
-			int res = static_cast<CStorjListOpData&>(*operations_.back()).ParseEntry(std::move(message.text[0]), message.text[1], std::move(message.text[2]), message.text[3]);
+			int res = static_cast<CStorjListOpData&>(*operations_.back()).ParseEntry(std::move(message.text[0]), message.text[1], message.text[3]);
 			if (res != FZ_REPLY_WOULDBLOCK) {
 				ResetOperation(res);
 			}
@@ -380,3 +346,15 @@ std::wstring CStorjControlSocket::QuoteFilename(std::wstring const& filename)
 {
 	return L"\"" + fz::replaced_substrings(filename, L"\"", L"\"\"") + L"\"";
 }
+
+void CStorjControlSocket::Push(std::unique_ptr<COpData> && pNewOpData)
+{
+	CControlSocket::Push(std::move(pNewOpData));
+	if (operations_.size() == 1 && operations_.back()->opId != Command::connect) {
+		if (!process_) {
+			std::unique_ptr<COpData> connOp = std::make_unique<CStorjConnectOpData>(*this);
+			connOp->topLevelOperation_ = true;
+			CControlSocket::Push(std::move(connOp));
+		}
+	}
+}
