@@ -21,10 +21,9 @@
 
 #include "commands.h"
 #include "local_path.h"
+#include "logging.h"
 #include "server.h"
 
-#include <logging.h>
-
 #include <libfilezilla/time.hpp>
 #include <libfilezilla/tls_info.hpp>
 
@@ -43,7 +42,7 @@ enum NotificationId
 	nId_logmsg,				// notification about new messages for the message log
 	nId_operation,			// operation reply codes
 	nId_connection,			// connection information: connects, disconnects, timeouts etc..
-	nId_transferstatus,		// transfer information: bytes transferes, transfer speed and such
+	nId_transferstatus,		// transfer information: bytes transferred, transfer speed and such
 	nId_listing,			// directory listings
 	nId_asyncrequest,		// asynchronous request
 	nId_active,				// sent if data gets either received or sent
@@ -99,13 +98,15 @@ public:
 	{}
 
 	template<typename String>
-	CLogmsgNotification(logmsg::type t, String && m)
+	CLogmsgNotification(logmsg::type t, String && m, fz::datetime const& time)
 		: msg(std::forward<String>(m))
+		, time_(time)
 		, msgType(t)
 	{
 	}
 
 	std::wstring msg;
+	fz::datetime time_;
 	logmsg::type msgType{logmsg::status}; // Type of message, see logging.h for details
 };
 
@@ -114,8 +115,14 @@ public:
 class COperationNotification final : public CNotificationHelper<nId_operation>
 {
 public:
-	int nReplyCode{};
-	Command commandId{Command::none};
+	COperationNotification() = default;
+	COperationNotification(int replyCode, Command commandId)
+		: replyCode_(replyCode)
+		, commandId_(commandId)
+	{}
+
+	int replyCode_{};
+	Command commandId_{Command::none};
 };
 
 // You get this type of notification everytime a directory listing has been
