@@ -29,6 +29,7 @@ public:
 	CServerPath& operator=(CServerPath const& op) = default;
 	CServerPath& operator=(CServerPath && op) noexcept = default;
 
+	explicit operator bool() const { return !empty(); }
 	bool empty() const { return !m_data; }
 	void clear();
 
@@ -74,6 +75,8 @@ public:
 	bool AddSegment(std::wstring const& segment);
 
 	size_t SegmentCount() const;
+
+	static CServerPath GetChanged(CServerPath const& oldPath, CServerPath const& newPath, std::wstring const& newSubdir);
 private:
 	bool IsSeparator(wchar_t c) const;
 
