@@ -1,7 +1,10 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "listctrlex.h"
 #include "filezillaapp.h"
+
 #include <wx/renderer.h>
+#include <wx/statbox.h>
+
 #include "Options.h"
 #include "dialogex.h"
 #ifdef __WXMSW__
@@ -430,19 +433,19 @@ void wxListCtrlEx::ShowColumn(unsigned int col, bool show)
 	}
 }
 
-void wxListCtrlEx::LoadColumnSettings(int widthsOptionId, int visibilityOptionId, int sortOptionId)
+void wxListCtrlEx::LoadColumnSettings(interfaceOptions widthsOptionId, interfaceOptions visibilityOptionId, interfaceOptions sortOptionId)
 {
 	wxASSERT(!GetColumnCount());
 
-	if (widthsOptionId != -1) {
+	if (widthsOptionId != OPTIONS_NUM) {
 		ReadColumnWidths(widthsOptionId);
 	}
 
 	delete [] m_pVisibleColumnMapping;
 	m_pVisibleColumnMapping = new unsigned int[m_columnInfo.size()];
 
-	if (visibilityOptionId != -1) {
-		wxString visibleColumns = COptions::Get()->GetOption(visibilityOptionId);
+	if (visibilityOptionId != OPTIONS_NUM) {
+		wxString visibleColumns = COptions::Get()->get_string(visibilityOptionId);
 		if (visibleColumns.Len() >= m_columnInfo.size()) {
 			for (unsigned int i = 0; i < m_columnInfo.size(); ++i) {
 				if (!m_columnInfo[i].fixed) {
@@ -452,8 +455,8 @@ void wxListCtrlEx::LoadColumnSettings(int widthsOptionId, int visibilityOptionId
 		}
 	}
 
-	if (sortOptionId != -1) {
-		auto tokens = fz::strtok(COptions::Get()->GetOption(sortOptionId), L",");
+	if (sortOptionId != OPTIONS_NUM) {
+		auto tokens = fz::strtok(COptions::Get()->get_string(sortOptionId), L",");
 
 		if (tokens.size() >= m_columnInfo.size()) {
 			unsigned int *order = new unsigned int[m_columnInfo.size()];
@@ -500,13 +503,13 @@ void wxListCtrlEx::LoadColumnSettings(int widthsOptionId, int visibilityOptionId
 	CreateVisibleColumnMapping();
 }
 
-void wxListCtrlEx::SaveColumnSettings(int widthsOptionId, int visibilityOptionId, int sortOptionId)
+void wxListCtrlEx::SaveColumnSettings(interfaceOptions widthsOptionId, interfaceOptions visibilityOptionId, interfaceOptions sortOptionId)
 {
-	if (widthsOptionId != -1) {
+	if (widthsOptionId != OPTIONS_NUM) {
 		SaveColumnWidths(widthsOptionId);
 	}
 
-	if (visibilityOptionId != -1) {
+	if (visibilityOptionId != OPTIONS_NUM) {
 		std::wstring visibleColumns;
 		for (unsigned int i = 0; i < m_columnInfo.size(); ++i) {
 			if (m_columnInfo[i].shown) {
@@ -516,10 +519,10 @@ void wxListCtrlEx::SaveColumnSettings(int widthsOptionId, int visibilityOptionId
 				visibleColumns += L"0";
 			}
 		}
-		COptions::Get()->SetOption(visibilityOptionId, visibleColumns);
+		COptions::Get()->set(visibilityOptionId, visibleColumns);
 	}
 
-	if (sortOptionId != -1) {
+	if (sortOptionId != OPTIONS_NUM) {
 		std::wstring order;
 		for (unsigned int i = 0; i < m_columnInfo.size(); ++i) {
 			if (i) {
@@ -527,11 +530,11 @@ void wxListCtrlEx::SaveColumnSettings(int widthsOptionId, int visibilityOptionId
 			}
 			order += fz::to_wstring(m_columnInfo[i].order);
 		}
-		COptions::Get()->SetOption(sortOptionId, order);
+		COptions::Get()->set(sortOptionId, order);
 	}
 }
 
-bool wxListCtrlEx::ReadColumnWidths(unsigned int optionId)
+bool wxListCtrlEx::ReadColumnWidths(interfaceOptions optionId)
 {
 	wxASSERT(!GetColumnCount());
 
@@ -543,7 +546,7 @@ bool wxListCtrlEx::ReadColumnWidths(unsigned int optionId)
 	}
 
 
-	auto tokens = fz::strtok(COptions::Get()->GetOption(optionId), L" ");
+	auto tokens = fz::strtok(COptions::Get()->get_string(optionId), L" ");
 
 	auto const count = std::min(tokens.size(), m_columnInfo.size());
 	for (size_t i = 0; i < count; ++i) {
@@ -556,7 +559,7 @@ bool wxListCtrlEx::ReadColumnWidths(unsigned int optionId)
 	return true;
 }
 
-void wxListCtrlEx::SaveColumnWidths(unsigned int optionId)
+void wxListCtrlEx::SaveColumnWidths(interfaceOptions optionId)
 {
 	const unsigned int count = m_columnInfo.size();
 
@@ -580,7 +583,7 @@ void wxListCtrlEx::SaveColumnWidths(unsigned int optionId)
 	}
 	widths.RemoveLast();
 
-	COptions::Get()->SetOption(optionId, widths.ToStdWstring());
+	COptions::Get()->set(optionId, widths.ToStdWstring());
 }
 
 
@@ -686,32 +689,81 @@ void wxListCtrlEx::CreateVisibleColumnMapping()
 	}
 }
 
-class CColumnEditDialog : public wxDialogEx
+class CColumnEditDialog final : public wxDialogEx
 {
 public:
-	int *m_order;
-	DECLARE_EVENT_TABLE()
+	std::vector<int> order_;
+	wxCheckListBox * list_box_{};
+
+	bool Create(wxWindow* parent)
+	{
+		if (!wxDialogEx::Create(parent, -1, _("Column setup"))) {
+			return false;
+		}
+
+		auto& lay = layout();
+		auto main = lay.createMain(this, 1);
+
+		{
+			auto [box, inner] = lay.createStatBox(main, _("Visible columns"), 1);
+			inner->AddGrowableCol(0);
+			inner->Add(new wxStaticText(box, -1, _("&Select the columns that should be displayed:")));
+
+			auto row = lay.createFlex(2);
+			row->AddGrowableCol(0);
+			inner->Add(row, lay.grow);
+			list_box_ = new wxCheckListBox(box, -1);
+			list_box_->Bind(wxEVT_LISTBOX, &CColumnEditDialog::OnSelChanged, this);
+			list_box_->Bind(wxEVT_CHECKLISTBOX, &CColumnEditDialog::OnCheck, this);
+			row->Add(list_box_, lay.grow);
+
+			auto col = lay.createFlex(1);
+			col->AddGrowableCol(0);
+			row->Add(col);
+			up_ = new wxButton(box, -1, _("Move &up"));
+			up_->Bind(wxEVT_BUTTON, &CColumnEditDialog::OnUp, this);
+			up_->Disable();
+			col->Add(up_, lay.grow);
+			down_ = new wxButton(box, -1, _("Move &down"));
+			down_->Bind(wxEVT_BUTTON, &CColumnEditDialog::OnDown, this);
+			down_->Disable();
+			col->Add(down_, lay.grow);
+		}
+
+		auto buttons = lay.createButtonSizer(this, main, true);
+		auto ok = new wxButton(this, wxID_OK, _("OK"));
+		ok->SetDefault();
+		buttons->AddButton(ok);
+		auto cancel = new wxButton(this, wxID_CANCEL, _("Cancel"));
+		buttons->AddButton(cancel);
+		buttons->Realize();
+
+		Layout();
+		GetSizer()->Fit(this);
+
+		return true;
+	}
 
 protected:
+
+	wxButton* up_{};
+	wxButton* down_{};
+
 	void OnUp(wxCommandEvent&)
 	{
-		wxCheckListBox* pListBox = XRCCTRL(*this, "ID_ACTIVE", wxCheckListBox);
-		int sel = pListBox->GetSelection();
+		int sel = list_box_->GetSelection();
 		if (sel < 2) {
 			return;
 		}
 
-		int tmp;
-		tmp = m_order[sel - 1];
-		m_order[sel - 1] = m_order[sel];
-		m_order[sel] = tmp;
+		std::swap(order_[sel], order_[sel - 1]);
 
-		wxString name = pListBox->GetString(sel);
-		bool checked = pListBox->IsChecked(sel);
-		pListBox->Delete(sel);
-		pListBox->Insert(name, sel - 1);
-		pListBox->Check(sel - 1, checked);
-		pListBox->SetSelection(sel - 1);
+		wxString name = list_box_->GetString(sel);
+		bool checked = list_box_->IsChecked(sel);
+		list_box_->Delete(sel);
+		list_box_->Insert(name, sel - 1);
+		list_box_->Check(sel - 1, checked);
+		list_box_->SetSelection(sel - 1);
 
 		wxCommandEvent evt;
 		OnSelChanged(evt);
@@ -719,26 +771,22 @@ protected:
 
 	void OnDown(wxCommandEvent&)
 	{
-		wxCheckListBox* pListBox = XRCCTRL(*this, "ID_ACTIVE", wxCheckListBox);
-		int sel = pListBox->GetSelection();
+		int sel = list_box_->GetSelection();
 		if (sel < 1) {
 			return;
 		}
-		if (sel >= (int)pListBox->GetCount() - 1) {
+		if (sel >= (int)list_box_->GetCount() - 1) {
 			return;
 		}
 
-		int tmp;
-		tmp = m_order[sel + 1];
-		m_order[sel + 1] = m_order[sel];
-		m_order[sel] = tmp;
+		std::swap(order_[sel], order_[sel + 1]);
 
-		wxString name = pListBox->GetString(sel);
-		bool checked = pListBox->IsChecked(sel);
-		pListBox->Delete(sel);
-		pListBox->Insert(name, sel + 1);
-		pListBox->Check(sel + 1, checked);
-		pListBox->SetSelection(sel + 1);
+		wxString name = list_box_->GetString(sel);
+		bool checked = list_box_->IsChecked(sel);
+		list_box_->Delete(sel);
+		list_box_->Insert(name, sel + 1);
+		list_box_->Check(sel + 1, checked);
+		list_box_->SetSelection(sel + 1);
 
 		wxCommandEvent evt;
 		OnSelChanged(evt);
@@ -746,65 +794,52 @@ protected:
 
 	void OnSelChanged(wxCommandEvent&)
 	{
-		wxCheckListBox* pListBox = XRCCTRL(*this, "ID_ACTIVE", wxCheckListBox);
-		int sel = pListBox->GetSelection();
-		XRCCTRL(*this, "ID_UP", wxButton)->Enable(sel > 1);
-		XRCCTRL(*this, "ID_DOWN", wxButton)->Enable(sel > 0 && sel < (int)pListBox->GetCount() - 1);
+		int sel = list_box_->GetSelection();
+		up_->Enable(sel > 1);
+		down_->Enable(sel > 0 && sel < (int)list_box_->GetCount() - 1);
 	}
 
 	void OnCheck(wxCommandEvent& event)
 	{
 		if (!event.GetSelection() && !event.IsChecked()) {
-			wxCheckListBox* pListBox = XRCCTRL(*this, "ID_ACTIVE", wxCheckListBox);
-			pListBox->Check(0);
+			list_box_->Check(0);
 			wxMessageBoxEx(_("The filename column can neither be hidden nor moved."), _("Column properties"));
 		}
 	}
 };
 
-BEGIN_EVENT_TABLE(CColumnEditDialog, wxDialogEx)
-EVT_BUTTON(XRCID("ID_UP"), CColumnEditDialog::OnUp)
-EVT_BUTTON(XRCID("ID_DOWN"), CColumnEditDialog::OnDown)
-EVT_LISTBOX(wxID_ANY, CColumnEditDialog::OnSelChanged)
-EVT_CHECKLISTBOX(wxID_ANY, CColumnEditDialog::OnCheck)
-END_EVENT_TABLE()
-
 void wxListCtrlEx::ShowColumnEditor()
 {
 	CColumnEditDialog dlg;
 
-	if (!dlg.Load(this, _T("ID_COLUMN_SETUP"))) {
-		wxBell();
+	if (!dlg.Create(this)) {
 		return;
 	}
-
-	wxCheckListBox* pListBox = XRCCTRL(dlg, "ID_ACTIVE", wxCheckListBox);
-
-	dlg.m_order = new int[m_columnInfo.size()];
-	for (unsigned int j = 0; j < m_columnInfo.size(); ++j) {
-		for (unsigned int i = 0; i < m_columnInfo.size(); ++i) {
+	
+	dlg.order_.resize(m_columnInfo.size());
+	for (size_t j = 0; j < m_columnInfo.size(); ++j) {
+		for (size_t i = 0; i < m_columnInfo.size(); ++i) {
 			if (m_columnInfo[i].order != j) {
 				continue;
 			}
-			dlg.m_order[j] = i;
-			pListBox->Append(m_columnInfo[i].name);
+			dlg.order_[j] = i;
+			dlg.list_box_->Append(m_columnInfo[i].name);
 			if (m_columnInfo[i].shown) {
-				pListBox->Check(j);
+				dlg.list_box_->Check(j);
 			}
 		}
 	}
-	wxASSERT(pListBox->GetCount() == m_columnInfo.size());
+	wxASSERT(dlg.list_box_->GetCount() == m_columnInfo.size());
 
 	dlg.GetSizer()->Fit(&dlg);
 
 	if (dlg.ShowModal() != wxID_OK) {
-		delete [] dlg.m_order;
 		return;
 	}
 
-	for (unsigned int i = 0; i < m_columnInfo.size(); ++i) {
-		int col = dlg.m_order[i];
-		bool isChecked = pListBox->IsChecked(i);
+	for (size_t i = 0; i < m_columnInfo.size(); ++i) {
+		int col = dlg.order_[i];
+		bool isChecked = dlg.list_box_->IsChecked(i);
 		if (!isChecked && !col) {
 			isChecked = true;
 			wxMessageBoxEx(_("The filename column cannot be hidden."));
@@ -815,8 +850,6 @@ void wxListCtrlEx::ShowColumnEditor()
 		}
 	}
 
-	delete [] dlg.m_order;
-
 	// Generic wxListCtrl needs manual refresh
 	Refresh();
 }
