@@ -1,16 +1,18 @@
-#include <filezilla.h>
+#include "filezilla.h"
 
 #include <libfilezilla/event_loop.hpp>
 #include <libfilezilla/format.hpp>
 #include <libfilezilla/iputils.hpp>
 #include <libfilezilla/translate.hpp>
 #include <libfilezilla/util.hpp>
-#include "engine_context.h"
-#include "netconfwizard.h"
-#include "Options.h"
+
+#include "../include/engine_context.h"
+#include "../include/externalipresolver.h"
+
 #include "dialogex.h"
 #include "filezillaapp.h"
-#include "externalipresolver.h"
+#include "netconfwizard.h"
+#include "Options.h"
 #include "xrc_helper.h"
 
 DECLARE_EVENT_TYPE(fzEVT_ON_EXTERNAL_IP_ADDRESS, -1)
@@ -86,7 +88,7 @@ bool CNetConfWizard::Load()
 
 	// Load values
 
-	switch (m_pOptions->GetOptionVal(OPTION_USEPASV))
+	switch (m_pOptions->get_int(OPTION_USEPASV))
 	{
 	default:
 	case 1:
@@ -97,9 +99,9 @@ bool CNetConfWizard::Load()
 		break;
 	}
 
-	XRCCTRL(*this, "ID_FALLBACK", wxCheckBox)->SetValue(m_pOptions->GetOptionVal(OPTION_ALLOW_TRANSFERMODEFALLBACK) != 0);
+	XRCCTRL(*this, "ID_FALLBACK", wxCheckBox)->SetValue(m_pOptions->get_int(OPTION_ALLOW_TRANSFERMODEFALLBACK) != 0);
 
-	switch (m_pOptions->GetOptionVal(OPTION_PASVREPLYFALLBACKMODE))
+	switch (m_pOptions->get_int(OPTION_PASVREPLYFALLBACKMODE))
 	{
 	default:
 	case 0:
@@ -109,7 +111,7 @@ bool CNetConfWizard::Load()
 		XRCCTRL(*this, "ID_PASSIVE_FALLBACK2", wxRadioButton)->SetValue(true);
 		break;
 	}
-	switch (m_pOptions->GetOptionVal(OPTION_EXTERNALIPMODE))
+	switch (m_pOptions->get_int(OPTION_EXTERNALIPMODE))
 	{
 	default:
 	case 0:
@@ -122,7 +124,7 @@ bool CNetConfWizard::Load()
 		XRCCTRL(*this, "ID_ACTIVEMODE3", wxRadioButton)->SetValue(true);
 		break;
 	}
-	switch (m_pOptions->GetOptionVal(OPTION_LIMITPORTS))
+	switch (m_pOptions->get_int(OPTION_LIMITPORTS))
 	{
 	default:
 	case 0:
@@ -132,11 +134,11 @@ bool CNetConfWizard::Load()
 		XRCCTRL(*this, "ID_ACTIVE_PORTMODE2", wxRadioButton)->SetValue(true);
 		break;
 	}
-	XRCCTRL(*this, "ID_ACTIVE_PORTMIN", wxTextCtrl)->SetValue(wxString::Format(_T("%d"), m_pOptions->GetOptionVal(OPTION_LIMITPORTS_LOW)));
-	XRCCTRL(*this, "ID_ACTIVE_PORTMAX", wxTextCtrl)->SetValue(wxString::Format(_T("%d"), m_pOptions->GetOptionVal(OPTION_LIMITPORTS_HIGH)));
-	XRCCTRL(*this, "ID_ACTIVEIP", wxTextCtrl)->SetValue(m_pOptions->GetOption(OPTION_EXTERNALIP));
-	XRCCTRL(*this, "ID_ACTIVERESOLVER", wxTextCtrl)->SetValue(m_pOptions->GetOption(OPTION_EXTERNALIPRESOLVER));
-	XRCCTRL(*this, "ID_NOEXTERNALONLOCAL", wxCheckBox)->SetValue(m_pOptions->GetOptionVal(OPTION_NOEXTERNALONLOCAL) != 0);
+	XRCCTRL(*this, "ID_ACTIVE_PORTMIN", wxTextCtrl)->SetValue(wxString::Format(_T("%d"), m_pOptions->get_int(OPTION_LIMITPORTS_LOW)));
+	XRCCTRL(*this, "ID_ACTIVE_PORTMAX", wxTextCtrl)->SetValue(wxString::Format(_T("%d"), m_pOptions->get_int(OPTION_LIMITPORTS_HIGH)));
+	XRCCTRL(*this, "ID_ACTIVEIP", wxTextCtrl)->SetValue(m_pOptions->get_string(OPTION_EXTERNALIP));
+	XRCCTRL(*this, "ID_ACTIVERESOLVER", wxTextCtrl)->SetValue(m_pOptions->get_string(OPTION_EXTERNALIPRESOLVER));
+	XRCCTRL(*this, "ID_NOEXTERNALONLOCAL", wxCheckBox)->SetValue(m_pOptions->get_int(OPTION_NOEXTERNALONLOCAL) != 0);
 
 	return true;
 }
@@ -856,27 +858,27 @@ void CNetConfWizard::OnFinish(wxWizardEvent&)
 		}
 	}
 
-	m_pOptions->SetOption(OPTION_USEPASV, XRCCTRL(*this, "ID_PASSIVE", wxRadioButton)->GetValue() ? 1 : 0);
-	m_pOptions->SetOption(OPTION_ALLOW_TRANSFERMODEFALLBACK, XRCCTRL(*this, "ID_FALLBACK", wxCheckBox)->GetValue() ? 1 : 0);
+	m_pOptions->set(OPTION_USEPASV, XRCCTRL(*this, "ID_PASSIVE", wxRadioButton)->GetValue() ? 1 : 0);
+	m_pOptions->set(OPTION_ALLOW_TRANSFERMODEFALLBACK, XRCCTRL(*this, "ID_FALLBACK", wxCheckBox)->GetValue() ? 1 : 0);
 
-	m_pOptions->SetOption(OPTION_PASVREPLYFALLBACKMODE, XRCCTRL(*this, "ID_PASSIVE_FALLBACK1", wxRadioButton)->GetValue() ? 0 : 1);
+	m_pOptions->set(OPTION_PASVREPLYFALLBACKMODE, XRCCTRL(*this, "ID_PASSIVE_FALLBACK1", wxRadioButton)->GetValue() ? 0 : 1);
 
 	if (XRCCTRL(*this, "ID_ACTIVEMODE1", wxRadioButton)->GetValue()) {
-		m_pOptions->SetOption(OPTION_EXTERNALIPMODE, 0);
+		m_pOptions->set(OPTION_EXTERNALIPMODE, 0);
 	}
 	else {
-		m_pOptions->SetOption(OPTION_EXTERNALIPMODE, XRCCTRL(*this, "ID_ACTIVEMODE2", wxRadioButton)->GetValue() ? 1 : 2);
+		m_pOptions->set(OPTION_EXTERNALIPMODE, XRCCTRL(*this, "ID_ACTIVEMODE2", wxRadioButton)->GetValue() ? 1 : 2);
 	}
 
-	m_pOptions->SetOption(OPTION_LIMITPORTS, XRCCTRL(*this, "ID_ACTIVE_PORTMODE1", wxRadioButton)->GetValue() ? 0 : 1);
+	m_pOptions->set(OPTION_LIMITPORTS, XRCCTRL(*this, "ID_ACTIVE_PORTMODE1", wxRadioButton)->GetValue() ? 0 : 1);
 
 	long tmp;
-	XRCCTRL(*this, "ID_ACTIVE_PORTMIN", wxTextCtrl)->GetValue().ToLong(&tmp); m_pOptions->SetOption(OPTION_LIMITPORTS_LOW, tmp);
-	XRCCTRL(*this, "ID_ACTIVE_PORTMAX", wxTextCtrl)->GetValue().ToLong(&tmp); m_pOptions->SetOption(OPTION_LIMITPORTS_HIGH, tmp);
+	XRCCTRL(*this, "ID_ACTIVE_PORTMIN", wxTextCtrl)->GetValue().ToLong(&tmp); m_pOptions->set(OPTION_LIMITPORTS_LOW, tmp);
+	XRCCTRL(*this, "ID_ACTIVE_PORTMAX", wxTextCtrl)->GetValue().ToLong(&tmp); m_pOptions->set(OPTION_LIMITPORTS_HIGH, tmp);
 
-	m_pOptions->SetOption(OPTION_EXTERNALIP, XRCCTRL(*this, "ID_ACTIVEIP", wxTextCtrl)->GetValue().ToStdWstring());
-	m_pOptions->SetOption(OPTION_EXTERNALIPRESOLVER, XRCCTRL(*this, "ID_ACTIVERESOLVER", wxTextCtrl)->GetValue().ToStdWstring());
-	m_pOptions->SetOption(OPTION_NOEXTERNALONLOCAL, XRCCTRL(*this, "ID_NOEXTERNALONLOCAL", wxCheckBox)->GetValue());
+	m_pOptions->set(OPTION_EXTERNALIP, XRCCTRL(*this, "ID_ACTIVEIP", wxTextCtrl)->GetValue().ToStdWstring());
+	m_pOptions->set(OPTION_EXTERNALIPRESOLVER, XRCCTRL(*this, "ID_ACTIVERESOLVER", wxTextCtrl)->GetValue().ToStdWstring());
+	m_pOptions->set(OPTION_NOEXTERNALONLOCAL, XRCCTRL(*this, "ID_NOEXTERNALONLOCAL", wxCheckBox)->GetValue());
 }
 
 int CNetConfWizard::CreateListenSocket()
