@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "Options.h"
 #include "queue.h"
 #include "queueview_failed.h"
@@ -7,6 +7,8 @@
 #include "timeformatting.h"
 #include "themeprovider.h"
 
+#include <wx/filedlg.h>
+
 CQueueItem::CQueueItem(CQueueItem* parent)
 	: m_parent(parent)
 {
@@ -327,10 +329,10 @@ bool CFileItem::TryRemoveAll()
 	return false;
 }
 
-void CFileItem::SetTargetFile(wxString const& file)
+void CFileItem::SetTargetFile(std::wstring const& file)
 {
 	if (!file.empty() && file != m_sourceFile) {
-		m_targetFile = fz::sparse_optional<std::wstring>(fz::to_wstring(file));
+		m_targetFile = fz::sparse_optional<std::wstring>(file);
 	}
 	else {
 		m_targetFile.clear();
@@ -398,9 +400,9 @@ void CFolderItem::SetActive(const bool active)
 	}
 }
 
-CServerItem::CServerItem(ServerWithCredentials const& server)
+CServerItem::CServerItem(Site const& site)
 	: m_activeCount(0)
-	, server_(server)
+	, site_(site)
 {
 }
 
@@ -408,14 +410,9 @@ CServerItem::~CServerItem()
 {
 }
 
-ServerWithCredentials const& CServerItem::GetServer() const
-{
-	return server_;
-}
-
 wxString CServerItem::GetName() const
 {
-	return server_.Format(ServerFormat::with_user_and_optional_port);
+	return site_.Format(ServerFormat::with_user_and_optional_port);
 }
 
 void CServerItem::AddChild(CQueueItem* pItem)
@@ -719,7 +716,7 @@ void CServerItem::QueueImmediateFile(CFileItem* pItem)
 void CServerItem::SaveItem(pugi::xml_node& element) const
 {
 	auto server_node = element.append_child("Server");
-	SetServer(server_node, server_);
+	SetServer(server_node, site_);
 
 	for (auto iter = m_children.cbegin() + m_removed_at_front; iter != m_children.cend(); ++iter) {
 		(*iter)->SaveItem(server_node);
@@ -1172,7 +1169,7 @@ void CQueueViewBase::UpdateSelections_ItemRangeAdded(int added, int count)
 			SetItemState(itemsToSelect.front(), wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
 			itemsToSelect.pop_front();
 		}
-		if (itemsToSelect.front() == item) {
+		if (!itemsToSelect.empty() && itemsToSelect.front() == item) {
 			itemsToSelect.pop_front();
 		}
 		else {
@@ -1295,25 +1292,25 @@ void CQueueViewBase::CreateColumns(std::vector<ColumnId> const& extraColumns)
 		AddQueueColumn(id);
 	}
 
-	LoadColumnSettings(OPTION_QUEUE_COLUMN_WIDTHS, -1, -1);
+	LoadColumnSettings(OPTION_QUEUE_COLUMN_WIDTHS, OPTIONS_NUM, OPTIONS_NUM);
 }
 
-CServerItem* CQueueViewBase::GetServerItem(ServerWithCredentials const& server)
+CServerItem* CQueueViewBase::GetServerItem(Site const& site)
 {
 	for (auto iter = m_serverList.begin(); iter != m_serverList.end(); ++iter) {
-		if ((*iter)->GetServer() == server) {
+		if ((*iter)->GetSite() == site) {
 			return *iter;
 		}
 	}
 	return NULL;
 }
 
-CServerItem* CQueueViewBase::CreateServerItem(ServerWithCredentials const& server)
+CServerItem* CQueueViewBase::CreateServerItem(Site const& site)
 {
-	CServerItem* pItem = GetServerItem(server);
+	CServerItem* pItem = GetServerItem(site);
 
 	if (!pItem) {
-		pItem = new CServerItem(server);
+		pItem = new CServerItem(site);
 		m_serverList.push_back(pItem);
 		++m_itemCount;
 
