@@ -1,11 +1,12 @@
-#include <filezilla.h>
+#include "filezilla.h"
 
 #if FZ_MANUALUPDATECHECK
 
 #include "buildinfo.h"
-#include "updater.h"
-#include "Options.h"
 #include "file_utils.h"
+#include "Options.h"
+#include "updater.h"
+#include "serverdata.h"
 #include <string>
 
 #ifdef __WXMSW__
@@ -61,10 +62,10 @@ PqQuF7sJR6POArUVYkRD/2LIWsB7\n\
 
 void version_information::update_available()
 {
-	if (!nightly_.url_.empty() && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_CHECKBETA) == 2) {
+	if (!nightly_.url_.empty() && COptions::Get()->get_int(OPTION_UPDATECHECK_CHECKBETA) == 2) {
 		available_ = nightly_;
 	}
-	else if (!beta_.version_.empty() && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_CHECKBETA) != 0) {
+	else if (!beta_.version_.empty() && COptions::Get()->get_int(OPTION_UPDATECHECK_CHECKBETA) != 0) {
 		available_ = beta_;
 	}
 	else if (!stable_.version_.empty()) {
@@ -91,8 +92,8 @@ void CUpdater::Init()
 		return;
 	}
 
-	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_DISABLEUPDATECHECK) != 0 || !LongTimeSinceLastCheck()) {
-		raw_version_information_ = COptions::Get()->GetOption(OPTION_UPDATECHECK_NEWVERSION);
+	if (COptions::Get()->get_int(OPTION_DEFAULT_DISABLEUPDATECHECK) != 0 || !LongTimeSinceLastCheck()) {
+		raw_version_information_ = COptions::Get()->get_string(OPTION_UPDATECHECK_NEWVERSION);
 	}
 
 	UpdaterState s = ProcessFinishedData(FZ_AUTOUPDATECHECK);
@@ -128,7 +129,7 @@ void CUpdater::AutoRunIfNeeded()
 {
 #if FZ_AUTOUPDATECHECK
 	if (state_ == UpdaterState::failed || state_ == UpdaterState::idle || state_ == UpdaterState::newversion_stale) {
-		if (!COptions::Get()->GetOptionVal(OPTION_DEFAULT_DISABLEUPDATECHECK) && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK) != 0) {
+		if (!COptions::Get()->get_int(OPTION_DEFAULT_DISABLEUPDATECHECK) && COptions::Get()->get_int(OPTION_UPDATECHECK) != 0) {
 			if (LongTimeSinceLastCheck()) {
 				Run(false);
 			}
@@ -158,7 +159,7 @@ void CUpdater::RunIfNeeded()
 
 bool CUpdater::LongTimeSinceLastCheck() const
 {
-	std::wstring const lastCheckStr = COptions::Get()->GetOption(OPTION_UPDATECHECK_LASTDATE);
+	std::wstring const lastCheckStr = COptions::Get()->get_string(OPTION_UPDATECHECK_LASTDATE);
 	if (lastCheckStr.empty()) {
 		return true;
 	}
@@ -177,7 +178,7 @@ bool CUpdater::LongTimeSinceLastCheck() const
 
 	int days = 1;
 	if (!CBuildInfo::IsUnstable()) {
-		days = COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_INTERVAL);
+		days = COptions::Get()->get_int(OPTION_UPDATECHECK_INTERVAL);
 	}
 	return span.get_days() >= days;
 }
@@ -240,7 +241,7 @@ fz::uri CUpdater::GetUrl()
 		qs["cpuid"] = cpuCaps;
 	}
 
-	std::wstring const lastVersion = COptions::Get()->GetOption(OPTION_UPDATECHECK_LASTVERSION);
+	std::wstring const lastVersion = COptions::Get()->get_string(OPTION_UPDATECHECK_LASTVERSION);
 	if (lastVersion != CBuildInfo::GetVersion()) {
 		qs["initial"] = "1";
 	}
@@ -252,8 +253,7 @@ fz::uri CUpdater::GetUrl()
 		qs["manual"] = "1";
 	}
 
-	wxString v;
-	if (wxGetEnv(_T("FZUPDATETEST"), &v) && v == _T("1")) {
+	if (GetEnv("FZUPDATETEST") == L"1") {
 		qs["test"] = "1";
 	}
 	uri.query_ = qs.to_string(true);
@@ -270,7 +270,7 @@ bool CUpdater::Run(bool manual)
 	}
 
 	auto const t = fz::datetime::now();
-	COptions::Get()->SetOption(OPTION_UPDATECHECK_LASTDATE, t.format(L"%Y-%m-%d %H:%M:%S", fz::datetime::utc));
+	COptions::Get()->set(OPTION_UPDATECHECK_LASTDATE, t.format(L"%Y-%m-%d %H:%M:%S", fz::datetime::utc));
 
 	local_file_.clear();
 	log_ = fz::sprintf(_("Started update check on %s\n"), t.format(L"%Y-%m-%d %H:%M:%S", fz::datetime::local));
@@ -416,8 +416,8 @@ void CUpdater::ProcessNotification(std::unique_ptr<CNotification> && notificatio
 						auto const& ca = certs.back();
 						std::vector<uint8_t> ca_data = ca.get_raw_data();
 
-						std::string updater_root = fz::base64_decode(s_update_cert);
-						if (ca_data.size() == updater_root.size() && !memcmp(&ca_data[0], updater_root.c_str(), ca_data.size())) {
+						auto const updater_root = fz::base64_decode(s_update_cert);
+						if (ca_data == updater_root) {
 							certNotification.trusted_ = true;
 						}
 					}
@@ -507,8 +507,8 @@ void CUpdater::ProcessOperation(COperationNotification const& operation)
 
 	UpdaterState s = UpdaterState::failed;
 
-	int res = operation.nReplyCode;
-	if (res == FZ_REPLY_OK || (operation.commandId == Command::disconnect && res & FZ_REPLY_DISCONNECTED)) {
+	int res = operation.replyCode_;
+	if (res == FZ_REPLY_OK || (operation.commandId_ == Command::disconnect && res & FZ_REPLY_DISCONNECTED)) {
 		pending_commands_.pop_front();
 		res = ContinueDownload();
 		if (res == FZ_REPLY_WOULDBLOCK) {
@@ -522,7 +522,7 @@ void CUpdater::ProcessOperation(COperationNotification const& operation)
 		}
 	}
 	else if (state_ == UpdaterState::checking) {
-		COptions::Get()->SetOption(OPTION_UPDATECHECK_LASTVERSION, CBuildInfo::GetVersion());
+		COptions::Get()->set(OPTION_UPDATECHECK_LASTVERSION, CBuildInfo::GetVersion());
 		s = ProcessFinishedData(true);
 	}
 	else {
@@ -603,7 +603,7 @@ void CUpdater::ProcessData(CDataNotification& dataNotification)
 	size_t len;
 	char* data = dataNotification.Detach(len);
 
-	if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+	if (COptions::Get()->get_int(OPTION_LOGGING_DEBUGLEVEL) == 4) {
 		log_ += fz::sprintf(_T("ProcessData %u\n"), len);
 	}
 
@@ -660,7 +660,7 @@ void CUpdater::ParseData()
 			version_information_.changelog_ = raw_version_information;
 			fz::trim(version_information_.changelog_);
 
-			if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+			if (COptions::Get()->get_int(OPTION_LOGGING_DEBUGLEVEL) == 4) {
 				log_ += fz::sprintf(L"Changelog: %s\n", version_information_.changelog_);
 			}
 			break;
@@ -668,7 +668,7 @@ void CUpdater::ParseData()
 
 		std::wstring const& type = tokens[0];
 		if (tokens.size() < 2) {
-			if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+			if (COptions::Get()->get_int(OPTION_LOGGING_DEBUGLEVEL) == 4) {
 				log_ += fz::sprintf(L"Skipping line with one token of type %s\n", type);
 			}
 			continue;
@@ -676,7 +676,21 @@ void CUpdater::ParseData()
 
 		if (type == L"resources") {
 			if (UpdatableBuild()) {
-				version_information_.resources_ = tokens[1];
+				version_information_.resources_[resource_type::update_dialog] = tokens[1];
+			}
+			continue;
+		}
+		else if (type == "resource") {
+			if (tokens.size() >= 3) {
+				std::wstring resource;
+				for (size_t i = 2; i < tokens.size(); ++i) {
+					if (!resource.empty()) {
+						resource += ' ';
+					}
+					resource += tokens[i];
+				}
+				auto x = fz::to_integral<resource_type>(tokens[1]);
+				version_information_.resources_[fz::to_integral<resource_type>(tokens[1])] = std::move(resource);
 			}
 			continue;
 		}
@@ -695,7 +709,7 @@ void CUpdater::ParseData()
 				auto const& token = tokens[i];
 				if (token.substr(0, 4) == "sig:") {
 					auto const& sig = token.substr(4);
-					auto raw_sig = fz::base64_decode(fz::to_utf8(sig));
+					auto raw_sig = fz::base64_decode_s(fz::to_utf8(sig));
 
 					if (!raw_sig.empty()) {
 						auto const pub = fz::public_verification_key::from_base64("xrjuitldZT7pvIhK9q1GVNfptrepB/ctt5aK1QO5RaI");
@@ -716,7 +730,7 @@ void CUpdater::ParseData()
 		if (type == L"nightly") {
 			fz::datetime nightlyDate(versionOrDate, fz::datetime::utc);
 			if (nightlyDate.empty()) {
-				if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+				if (COptions::Get()->get_int(OPTION_LOGGING_DEBUGLEVEL) == 4) {
 					log_ += L"Could not parse nightly date\n";
 				}
 				continue;
@@ -724,7 +738,7 @@ void CUpdater::ParseData()
 
 			fz::datetime buildDate = CBuildInfo::GetBuildDate();
 			if (buildDate.empty() || nightlyDate.empty() || nightlyDate <= buildDate) {
-				if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+				if (COptions::Get()->get_int(OPTION_LOGGING_DEBUGLEVEL) == 4) {
 					log_ += L"Nightly isn't newer\n";
 				}
 				continue;
@@ -737,7 +751,7 @@ void CUpdater::ParseData()
 			}
 		}
 		else {
-			if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+			if (COptions::Get()->get_int(OPTION_LOGGING_DEBUGLEVEL) == 4) {
 				log_ += fz::sprintf(L"Skipping line with unknown type %s\n", type);
 			}
 			continue;
@@ -747,7 +761,7 @@ void CUpdater::ParseData()
 		b.version_ = versionOrDate;
 
 		if (tokens.size() < 6) {
-			if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+			if (COptions::Get()->get_int(OPTION_LOGGING_DEBUGLEVEL) == 4) {
 				log_ += fz::sprintf(L"Not parsing build line with only %d tokens", tokens.size());
 			}
 		}
@@ -758,7 +772,7 @@ void CUpdater::ParseData()
 			std::wstring const& hash = tokens[5];
 
 			if (GetFilename(url).empty()) {
-				if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+				if (COptions::Get()->get_int(OPTION_LOGGING_DEBUGLEVEL) == 4) {
 					log_ += fz::sprintf(L"Could not extract filename from URL: %s\n", url);
 				}
 				continue;
@@ -770,7 +784,7 @@ void CUpdater::ParseData()
 
 			auto const size = fz::to_integral<uint64_t>(sizestr);
 			if (!size) {
-				if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+				if (COptions::Get()->get_int(OPTION_LOGGING_DEBUGLEVEL) == 4) {
 					log_ += fz::sprintf(L"Could not parse size: %s\n", sizestr);
 				}
 				continue;
@@ -790,7 +804,7 @@ void CUpdater::ParseData()
 
 					if (!raw_sig.empty() || !raw_hash.empty()) {
 						auto const pub = fz::public_verification_key::from_base64("xrjuitldZT7pvIhK9q1GVNfptrepB/ctt5aK1QO5RaI");
-						valid_signature = fz::verify(std::string_view(reinterpret_cast<char const*>(raw_hash.data()), raw_hash.size()), raw_sig, pub);
+						valid_signature = fz::verify(raw_hash, raw_sig, pub);
 					}
 				}
 			}
@@ -831,7 +845,7 @@ void CUpdater::ParseData()
 
 	version_information_.update_available();
 
-	COptions::Get()->SetOption(OPTION_UPDATECHECK_NEWVERSION, raw_version_information_);
+	COptions::Get()->set(OPTION_UPDATECHECK_NEWVERSION, raw_version_information_);
 }
 
 void CUpdater::OnTimer(wxTimerEvent&)
@@ -997,4 +1011,15 @@ bool CUpdater::Busy() const
 {
 	return state_ == UpdaterState::checking || state_ == UpdaterState::newversion_downloading;
 }
+
+std::wstring CUpdater::GetResources(resource_type t) const
+{
+	std::wstring ret;
+	auto const it = version_information_.resources_.find(t);
+	if (it != version_information_.resources_.cend()) {
+		ret = it->second;
+	}
+	return ret;
+}
+
 #endif
