@@ -498,6 +498,16 @@ int sftp_put_file(char *fname, char *outfname, int restart)
             }
         }
 
+        if (toplevel_callback_pending() && !err && !eof) {
+            /* If we have pending callbacks, they might make
+             * xfer_upload_ready start to return true. So we should
+             * run them and then re-check xfer_upload_ready, before
+             * we go as far as waiting for an entire packet to
+             * arrive. */
+            run_toplevel_callbacks();
+            continue;
+        }
+
         if (!xfer_done(xfer)) {
             pktin = sftp_recv();
             ret = xfer_upload_gotpkt(xfer, pktin);
@@ -1257,7 +1267,7 @@ int sftp_cmd_chmod(struct sftp_command *cmd)
         return 0;
     }
 
-    int ret = sftp_action_chmod(ctx, cname);
+    sftp_action_chmod(ctx, cname);
     sfree(cname);
     return 0;
 }
@@ -2121,6 +2131,9 @@ static StripCtrlChars *stderr_scc;
 /*
  * Main program. Parse arguments etc.
  */
+#if defined(__MINGW32__)
+__declspec(dllexport) // This forces ld to not strip relocations so that ASLR can work on MSW.
+#endif
 int psftp_main(int argc, char *argv[])
 {
     int i, ret;
